@@ -129,9 +129,6 @@ class Node():
 
     def make_nodeproto(self):
         return onnx.helper.make_node(self.op_type, self.input, self.output, self.name, **self.attr)
-        self.proto.name = self.name
-        self.proto.op_type = self.op_type
-        return self.proto
 
     def shape_infer(self, intensors: []):
         faketensors = [_get_tensor(tensor) for tensor in intensors]
@@ -263,6 +260,9 @@ class DivNode(NpMathBase):
         self.op_mac = DIV_MACS
 
     def value_infer(self, intensors: []):
+        if intensors[0].dtype == intensors[1].dtype:
+            if intensors[0].dtype in [numpy.int64]:
+                return [intensors[0] // intensors[1]]
         return [intensors[0] / intensors[1]]
 
 
@@ -434,8 +434,8 @@ class NonMaxSuppressionNode(Node):
     def value_infer(self, intensors: []):
         if len(intensors) >= 3:
             max_output_boxes_per_class = int(intensors[2][0])
-            return [numpy.zeros((max_output_boxes_per_class, 3), dtype=numpy.int)]
-        return [numpy.zeros((200, 3), dtype=numpy.int)]
+            return [numpy.zeros((max_output_boxes_per_class, 3), dtype=numpy.int64)]
+        return [numpy.zeros((200, 3), dtype=numpy.int64)]
 
 
 @NODE_REGISTRY.register()
@@ -592,7 +592,7 @@ class GatherNode(Node):
 
     def value_infer(self, intensors: []):
         outtensors = []
-        out = numpy.take(intensors[0], intensors[1].astype(dtype=numpy.int), axis=self.axis)
+        out = numpy.take(intensors[0], intensors[1].astype(dtype=numpy.int64), axis=self.axis)
         outtensors.append(out)
         return outtensors
 
@@ -773,7 +773,7 @@ class ShapeNode(Node):
         return [newshape]
 
     def value_infer(self, intensors: [numpy.ndarray]):
-        return [numpy.array(_get_shape(intensors[0]), dtype=numpy.int)]
+        return [numpy.array(_get_shape(intensors[0]), dtype=numpy.int64)]
 
 
 @NODE_REGISTRY.register()
@@ -825,7 +825,6 @@ class UpsampleNode(ResizeNode):
     pass
 
 
-
 @NODE_REGISTRY.register()
 class PoolBase(Node):
     def __init__(self, nodeproto):
@@ -1775,6 +1774,7 @@ class SplitNode(Node):
             end += v
         return numpy.split(intensors[0], splitpos, axis)
 
+
 def create_node(n: onnx.NodeProto):
     node_class = NODE_REGISTRY.get(n.op_type + 'Node')
     if node_class != None:
